c-------------------------------------------------------------------
c          *****  program  wftables-1.0 (12.09.2006)  *****
c
c
c                1.0  PURPOSE OF THIS PROGRAM
c         1.1 to read  wave function from wftables.src source file
c         1.2 to write wave function to output file (ASCII and HTM format)
c         1.3 to calculate eigenvalue of wave function
c         1.4 to test the ortogonalization conditions
c
c
c
c                 2.0  INPUT AND OUTPUT FILES
c         INPUTS FILES:
c                         wftables.inp   - user input file
c                         wftables.src   - source file of wave functions
c         OUTPUTS FILES:
c                         wftables.out   - short output file (ASCII)
c                         wftables.lst   - full  output file (ASCII)
c                         wftables.htm   - output file in HTM format
c
c
c
c                 3.0  FORMAT OF USER INPUT FILE  (wftables.inp)
c            3.1   ne  nlnl  nspin  lres  z  key
c
c             ne -number of electrons of atom or ion  (ne=2)
c             nlnl  - configuration
c                    (nlnl=2020,2021,2121 or 10nl, where n=1 -,5  l=0,1,2)
c             nspin= (2*S+1) , where S - spin moment  (nspin=1,3)
c             lres - orbital moment                   (lres=0,1,2)
c             Z    - nuclear charge of ion            (Z=1.0 - 36.0)
c             key  - key of reference                 ( key=11 )
c
c
c
c                 4.0  FORMAT OF OUTPUT FILE  (wftables.lst)
c            4.1  the copy wtfables.inp file (TITLE CARD)
c            4.2  the wave function parameters (commentaries in wftables.htm)
c            4.3  BLANK
c            4.4  eigenvalue in a. u. (1 a.u.=2 Ry) and normalization factor
c            4.5  output data for test of ortogonalization conditions
c
c
c
c                 5.0  FORMAT OF OUTPUT FILE  (wftables.out)
c            5.1  BLANK
c            5.2  normalization factor and eigenvalue in a. u. (1 a.u.=2 Ry)
c            5.3  output data for test of ortogonalization conditions
c
c
c
c
c                 6.0  HISTORY
c         08.2006   wave functions of helium - like ions
c-----------------------------------------------------------------
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      parameter   ( jxne=3, jcmain=50 )
      common/fmain/ zt,gx(jcmain),ax(jxne,jcmain),
     &  ns(jxne,jcmain),ls(jxne,jcmain),kmain,ne,nspin,lres,iprint
      common/wmain/ zw,gw(jcmain),aw(jxne,jcmain),
     &  nw(jxne,jcmain),lw(jxne,jcmain),kwmax,new,nspinw,lresw
      open(15,file='wftables.lst',status='unknown')
      open(17,file='wftables.out',status='unknown')
c------------------- read input file ------------------
      open(11,file='wftables.inp',status='old')
      read(11,*)    ne,nlres,nspin,lres,zt,key
      close(11)
      write(15,20)  ne,nlres,nspin,lres,zt,key,eng
  20  format(5x,i2,2x,i6,2x,i2,2x,i2,2x,f5.0,3x,i4,f16.8)
c------------------ save initial wave function ------------
      call rd_wftab(ne,nlres,nspin,key,zt,kfind)
      if(kfind.le.0) goto 99
      kwmax=kmain
      do 1 n=1,kmain
      nw(1,n)=ns(1,n)
      lw(1,n)=ls(1,n)
      nw(2,n)=ns(2,n)
      lw(2,n)=ls(2,n)
      aw(1,n)=ax(1,n)
      aw(2,n)=ax(2,n)
      gw(n)=  gx(n)
      write(15,25) ns(1,n),ls(1,n),ns(2,n),ls(2,n),gx(n),
     &        ax(1,n),ax(2,n)
 25   format(2x,4i1,6x,e23.16,1x,f9.4,1x,f9.4,5x,e16.8)
  1   continue
c-------------- define lres, zt  --------------
c      zt=zz
      n1= nlres/1000
      l1=(nlres - n1*1000)/100
      n2=(nlres - n1*1000 - l1*100)/10
      l2= nlres - n1*1000 - l1*100 - n2*10
      lres=l1 + l2
      call vnort(vif)
c------------------- renormalization ------------------------
c      if(dabs(vif-1.0).gt.1.e-8)  then
c      do 7 n=1,kmain
c      gx(n) = gx(n)/dsqrt(vif)
c  7   continue
c      call vnort(vif)
c      write(15,26)  vif
c  26  format(/,10x,'Normalization of output wave function =',f16.10)
c      endif
c-------------------- calculate energy ------------------
      call rak
      call energ(eng,rnm)
      write(15,29) eng,rnm
      write(17,129) zt,nspin,nlres,nspin,nlres,rnm,eng
  129 format(1x,/,'     z=',f7.1,'   <',i1,i5,' / ',i1,i5,' > =  ',
     &   f13.10, 4x,f16.10)
   29 format(/,/,'   Energy=',f16.10,7x,f16.10,/,/)
      call outhtm(nlres,eng)
c---------------  test ortogonalization ------------------
      if(nlres.eq.1020 .or. nlres.eq.1030 .or. nlres.eq.1040 .or.
     &   nlres.eq.1050 .or. nlres.eq.2020)  then
      nlresx=1010
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.1030 .or. nlres.eq.1040 .or. nlres.eq.1050 .or.
     &  (nlres.eq.2020.and. zt.gt.1.01) )  then
      nlresx=1020
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.1040 .or.nlres.eq.1050 .or.
     &    (nlres.eq.2020.and.zt.gt.1.01)  )     then
      nlresx=1030
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.1050 .or. (nlres.eq.2020.and.zt.gt.1.01) )  then
      nlresx=1040
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.2020 .and.zt.gt.1.01 )  then
      nlresx=1050
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.1031 .or. nlres.eq.1041 .or.
     &   nlres.eq.1051 .or. nlres.eq.2021)  then
      nlresx=1021
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.1041.or.nlres.eq.1051 .or. nlres.eq.2021)  then
      nlresx=1031
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.1051 .or. nlres.eq.2021)  then
      nlresx=1041
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.2021)  then
      nlresx=1051
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif


      if(nlres.eq.1042.or.nlres.eq.1052 .or.
     &    (nlres.eq.2121.and. zt.gt.1.01))  then
      nlresx=1032
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

      if(nlres.eq.1052 .or. (nlres.eq.2121.and. zt.gt.1.01))  then
      nlresx=1042
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif


      if(nlres.eq.2121 .and. zt.gt.1.01 )  then
      nlresx=1052
      call rd_wftab(ne,nlresx,nspin,key,zt,kfind)
      call vnort(vif)
      write(15,41) zt,nspin,nlres,nspin,nlresx,vif
      write(17,41) zt,nspin,nlres,nspin,nlresx,vif
      endif

   41 format('     z=',f7.1,'   <',i1,i5,' / ',i1,i5,' > =  ',e12.4)
   99 close(15)
      stop
      end

      subroutine rd_wftab(nei,nlres,nspini,key,zz,kfind)
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      character h1*1,h80*80,title*80
      parameter   ( jxne=3, jcmain=50 )
      common/fmain/ zt,gx(jcmain),ax(jxne,jcmain),
     &  ns(jxne,jcmain),ls(jxne,jcmain),kmain,ne,nspin,lres,iprint
      dimension  nlx(jxne,jcmain)

      nn=0
      kfind=0
      open(12,file='wftables.src',status='old')
      read(12,20) title
  20  format(a80)
   1  read(12,21,err=9,end=9) h1,h80
  21  format(a1,a80)
      if(h1.eq.'c' .or. h1.eq.'C') goto 1

   2  read(12,*,err=9,end=9) nex,nlresx,nspinx,lresx,zx,keyx
      if( nex.ne.nei .or. nlresx.ne.nlres .or. nspinx.ne.nspini .or.
     &    keyx.ne.key .or. dabs(1.0-zx/zz).gt.0.01 ) then
   3       read(12,20) h80
           if(h80.ne.' ') goto 3
           goto 2
           else
   5  read(12,20) h80
      if(h80.eq.' ') goto 6
      backspace 12
      nn=nn+1
      read(12,*) (nlx(i,nn),i=1,nex),gx(nn),(ax(i,nn),i=1,nex)
      do 7 i=1,ne
      ns(i,nn)=nlx(i,nn)/10
      ls(i,nn)=nlx(i,nn) - 10 * ns(i,nn)
  7   continue
      goto 5
   6  kmain=nn
      if(kmain.ge.1) kfind=1
      endif

   9  close(12)
      if(kfind.le.0) write(* ,29) nei,nlres,nspini,key,zz
      if(kfind.le.0) write(15,29) nei,nlres,nspini,key,zz
  29  format(' error: no data   nei=',i2,'  nlres=',i6,
     &       '  nspin=',i1,'  key=',i3,'  zz=',f6.1 )
      return
      end

         subroutine  outhtm(nlres,eng)
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      parameter   ( jxne=3, jcmain=50 )
      character hl1*2, hl2*2, hm1*1,hm2*1, hlres*1, hname(36)*2
      character hsig*1,hmain*9
      common/fmain/ zt,gx(jcmain),ax(jxne,jcmain),
     &  ns(jxne,jcmain),ls(jxne,jcmain),kmain,ne,nspin,lres,iprint
      data  hname/' H','He','Li','Be',' B',' C',' N',' O',' F','Ne',
     &    'Na','Mg','Al','Si',' P',' S','Cl','Ar',' K','Ca',
     &    'Sc','Ti',' V','Cr','Mn','Fe','Co','Ni','Cu','Zn',
     &    'Ga','Ge','As','Se','Br','Kr'  /
      open(16,file='wftables.htm',status='unknown')
      n1= nlres/1000
      l1=(nlres - n1*1000)/100
      n2=(nlres - n1*1000 - l1*100)/10
      l2= nlres - n1*1000 - l1*100 - n2*10
      lres=l1 + l2
      if(nspin.eq.1.and.lres.eq.0)   hmain='he_s1.htm'
      if(nspin.eq.1.and.lres.eq.1)   hmain='he_p1.htm'
      if(nspin.eq.1.and.lres.eq.2)   hmain='he_d1.htm'
      if(nspin.eq.3.and.lres.eq.0)   hmain='he_s3.htm'
      if(nspin.eq.3.and.lres.eq.1)   hmain='he_p3.htm'
      if(nspin.eq.3.and.lres.eq.2)   hmain='he_d3.htm'
      hsig='+'
      hlres=' '
      if(lres.eq.0) hlres='S'
      if(lres.eq.1) hlres='P'
      if(lres.eq.2) hlres='D'
      if(lres.eq.3) hlres='F'
      if(lres.eq.4) hlres='G'
      if(lres.eq.5) hlres='H'


      hm1=' '
      if(l1.eq.0) hm1='s'
      if(l1.eq.1) hm1='p'
      if(l1.eq.2) hm1='d'
      if(l1.eq.3) hm1='f'
      if(l1.eq.4) hm1='g'
      if(l1.eq.5) hm1='h'
      hm2=' '
      if(l2.eq.0) hm2='s'
      if(l2.eq.1) hm2='p'
      if(l2.eq.2) hm2='d'
      if(l2.eq.3) hm2='f'
      if(l2.eq.4) hm2='g'
      if(l2.eq.5) hm2='h'
      write(16,31)  hmain
  31  format(
     &   '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0',
     &     ' Transitional//EN">',
     & /,'<HTML><HEAD><TITLE> Wave function of',
     &   ' hekium-like ions </TITLE>',
     & /,'<META http-equiv=Content-Type content="text/html;',
     &   ' charset=windows-1251">',
     & /,'<META content="MSHTML 6.00.2800.1170"',
     &    ' name=GENERATOR></HEAD>',
     & /,' <BODY bgColor=#fff8dc>',/,/,/,
     & '<p align="center" > <A href="../',a9,'"> <h3> RETURN',
     &  ' </h3> </a> </p>',/,
     & /, ' <p> <font size="7" color="green"><strong>' )

      write(16,33)  n1,hm1,n2,hm2,nspin,hlres
  33  format('(',i1,a1,i1,a1,') <sup> ',i1,'  </sup> ',a1,
     &         ' &nbsp; &nbsp; &nbsp;')

      izt=zt
      if(izt.eq.1) hsig='-'
      if(izt.eq.2) hsig=' '
      write(16,32)  izt,hname(izt),izt-2,hsig
  32  format('Z=',i2,' &nbsp;  &nbsp; &nbsp; ',a2,
     &        ' <sup> ',i2,a1,' </sup>',
     &  '</strong></font></p>',/,'<font size="5">',
     &  /,'<p align="left"> <b> Table 1. </b>',

c     &     'This project </p>',
     &  ' Novikov N.V. //  Wave Function Value Database',
     &  ' <A href="http://cdfe.sinp.msu.ru/services/wftables/',/,
     &   'FirstPage_eng.htm">',
     &   ' http://cdfe.sinp.msu.ru/services/wftables/  </a>  </p>',

     &  /,/, /,'<TABLE width=600 border=2 >',
     &  /,'<TR>',
     & /,'<TD align=center width=50 rowspan=2>      j   </TD>',
     & /,' <TD align=center width=130 colspan=2>  R <sub> n1,l1 ',
     &       '</sub>(&alpha;1,r)  </TD>',
     & /,'<TD align=center width=130 colspan=2>  R <sub> n2,l2  ',
     &     '</sub>(&alpha;2,r)  </TD>',
     & /,'   <TD align=center width=250 rowspan=2>      g   </TD>',
     & /,' </TR><TR>',
     & /,'  <TD align=center width=50>  n1 l1    </TD>',
     & /,'  <TD align=center width=80>   &alpha;1      </TD>',
     & /,'  <TD align=center width=50>  n2 l2   </TD>',
     & /,'  <TD align=center width=80>    &alpha;2   </TD>',
     & /,' </TR><TR>')



      write(16,20)  nspin,nlres,zt
 20   format('<!--  ',i2,2x,i4,3x,f9.4,'    -->')

      do 1 j=1,kmain
       nx1= ns(1,j)
       lx1= ls(1,j)
       nx2= ns(2,j)
       lx2= ls(2,j)
      hl1=' '
      if(lx1.eq.0) hl1='s'
      if(lx1.eq.1) hl1='p'
      if(lx1.eq.2) hl1='d'
      if(lx1.eq.3) hl1='f'
      if(lx1.eq.4) hl1='g'
      if(lx1.eq.5) hl1='h'
      hl2=' '
      if(lx2.eq.0) hl2='s'
      if(lx2.eq.1) hl2='p'
      if(lx2.eq.2) hl2='d'
      if(lx2.eq.3) hl2='f'
      if(lx2.eq.4) hl2='g'
      if(lx2.eq.5) hl2='h'
      write(16,701) j,nx1,hl1,ax(1,j),nx2,hl2,ax(2,j),gx(j)
 701  format(
     &   5x,'<TD align=center width=30> ',12x, i3    ,12x,'</TD>',/
     &   5x,'<TD align=center width=50> ',12x, i2,a1 ,12x,'</TD>',/
     &   5x,'<TD align=center width=80> ',9x,f9.4,9x,'</TD>',/
     &   5x,'<TD align=center width=50> ',12x, i2,a1 ,12x,'</TD>',/
     &   5x,'<TD align=center width=80> ',9x,f9.4,9x,'</TD>',/
     &   5x,'<TD align=center width=250>',1x,e24.16,2x,'</TD>',/,
     &   2x,'</TR><TR>' )
  1   continue
      write(16,29) eng,hmain
 29   format('</TR></TABLE>',/,'<p>   Energy=  ',f14.8,5x,'a.u.   </p>',
     & /,/, '<p align="center" > <A href="../',a9,'"> <h3> RETURN',
     &  ' </h3> </a> </p>',/,' </font>',/,'</BODY></HTML>')

      close(16)
      return
      end


         subroutine rak
c-------------- calculate coeffisients for v12 term -------------------
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      parameter   ( jxne=3, jcmain=50 )
      common/fmain/ zt,gx(jcmain),ax(jxne,jcmain),
     &  ns(jxne,jcmain),ls(jxne,jcmain),kmain,ne,nspin,lres,iprint
      common/cnsrak/  c9j(3,5,5,9,5,5)
      data  eps/1.e-12/

       do 10 n=1,3
       do 11 i1=1,5
       do 12 i2=1,5
       do 13 l=1,9
       do 14 j1=1,5
       do 15 j2=1,5
       c9j(n,i1,i2,l,j1,j2)=0.0
  15   continue
  14   continue
  13   continue
  12   continue
  11   continue
  10   continue

      do 3 i=1,kmain
      li1=ls(1,i)
      li2=ls(2,i)
            do 4 j=1,kmain
            lf1=ls(1,j)
            lf2=ls(2,j)

            lmin=iabs(li1-lf1)
            if(iabs(li1-lf2).lt.lmin) lmin=iabs(li1-lf2)
            if(iabs(li2-lf1).lt.lmin) lmin=iabs(li2-lf1)
            if(iabs(li2-lf2).lt.lmin) lmin=iabs(li2-lf2)

            lmax=li1+lf1
            if(li1+lf2.gt.lmax) lmax=li1+lf2
            if(li2+lf1.gt.lmax) lmax=li2+lf1
            if(li2+lf2.gt.lmax) lmax=li2+lf2

      do 5 l=lmin,lmax
      ctmp=dabs(c9j(lres+1,li1+1,li2+1,l+1,lf1+1,lf2+1))
      if(ctmp.lt.eps) c9j(lres+1, li1+1, li2+1, l+1, lf1+1, lf2+1 )=
     &        clrak( lres, li1,li2,l,lf1,lf2 )

      if(li1.ne.li2 )   then
      ctmp=dabs(c9j(lres+1,li2+1,li1+1,l+1,lf1+1,lf2+1))
      if(ctmp.lt.eps) c9j(lres+1, li2+1, li1+1, l+1, lf1+1, lf2+1 )=
     &        clrak( lres, li2,li1,l,lf1,lf2 )
      endif

      if(lf1.ne.lf2 )  then
      ctmp=dabs(c9j(lres+1,li1+1,li2+1,l+1,lf2+1,lf1+1))
      if(ctmp.lt.eps) c9j(lres+1, li1+1, li2+1, l+1, lf2+1, lf1+1 )=
     &        clrak( lres, li1,li2,l,lf2,lf1 )

      if(li1.ne.li2)  then
      ctmp=dabs(c9j(lres+1,li2+1,li1+1,l+1,lf2+1,lf1+1))
      if(ctmp.lt.eps) c9j(lres+1, li2+1, li1+1, l+1, lf2+1, lf1+1 )=
     &              clrak( lres, li2,li1,l,lf2,lf1 )
      endif
      endif
 5    continue
 4    continue
 3    continue
      return
      end

              function clrak(lres,lf1,lf2,l,li1,li2)
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      data  nol/0/, mres/0/,  eps/1.e-18/
      clrak=0.0
      cleb1=clb(li1,nol,l,nol,lf1,nol)
      cleb2=clb(lf2,nol,l,nol,li2,nol)
      if(dabs(cleb1*cleb2).lt.eps) return
c
      mumax=2*l+1
      m1max=2*li1+1
      m2max=2*li2+1
      n1max=2*lf1+1
      n2max=2*lf2+1
c
      summu=0.0
      do 1 mmu=1,mumax
      mu=-l+mmu-1
c
      do 2 mm1=1,m1max
      m1=-li1+mm1-1
c
      do 3 mm2=1,m2max
      m2=-li2+mm2-1
      cl1=clb(li1,m1,li2,m2,lres,mres)
c
      do 4 nn1=1,n1max
      n1=-lf1+nn1-1
      cl3=clb(li1,m1, l ,mu,lf1 ,n1  )
c
      do 5 nn2=1,n2max
      n2=-lf2+nn2-1
      cl2=clb(lf1,n1,lf2,n2,lres,mres)
      cl4=clb(lf2,n2, l ,mu,li2 ,m2  )
      summu=summu + cl1*cl2*cl3*cl4
 5           continue
 4           continue
 3           continue
 2           continue
 1           continue
      const=dsqrt( dfloat( (2*li1+1)*(2*lf2+1) )    )
     &         / dsqrt(  dfloat( (2*lf1+1)*(2*li2+1) ) )
      clrak=summu*cleb1*cleb2*const
c  9   write(* ,21) lres,lf1,lf2,l,li1,li2,clrak
c      write(15,21) lres,lf1,lf2,l,li1,li2,clrak
c  21  format('  lres='i2,2x,i2,i3,5x,'l=',i2,5x,i2,i3,5x,e23.16)
      return
      end
           function  sr(ll,n1,z1,n2,z2)
c----------------- calculate integral dr1, dr2  from ----------------
c  r1**(2+n1) *r2**(2+n2) * exp(-z1*r1-z2*r2) * (r<)**ll/(r>)**(ll+1)
c--------------------------------------------------------------------
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      sum1=0.0
      jmax1=n1+ll+2
      f1=fn(n1+ll+2)
      do 1 jj1=1,jmax1
      j1=jj1-1
      sum1=sum1+f1/fn(n1+ll+2-j1) * 1./(z1**(j1+1))
     &           * fn(n1+n2+3-j1)/((z1+z2)**(n1+n2+4-j1))
   1  continue
c-
      if(ll.ge.n2+2)  then
      sum2=f1/(z1**(n1+ll+3)) * dlog(1.+z1/z2)
      else
      sum2=f1*fn(n2-ll+1)/(z1**(n1+ll+3))
     & *(  1./(z2**(n2-ll+2)) -1./((z1+z2)**(n2-ll+2)) )
      endif
c--
      sum3=0.0
      jmax2=n2+ll+2
      f2=fn(n2+ll+2)
      do 2 jj2=1,jmax2
      j2=jj2-1
      sum3=sum3+f2/fn(n2+ll+2-j2) * 1./(z2**(j2+1))
     &           * fn(n1+n2+3-j2)/((z1+z2)**(n1+n2+4-j2))
   2  continue
      if(ll.ge.n1+2)  then
      sum4=f2/(z2**(n2+ll+3)) * dlog(1.+z2/z1)
      else
      sum4=f2*fn(n1-ll+1)/(z2**(n2+ll+3))
     & *(  1./(z1**(n1-ll+2)) -1./((z1+z2)**(n1-ll+2)) )
      endif
      sr= - sum1 + sum2 - sum3 + sum4
c      write(15,21) ll,n1,z1,n2,z2,sr
c 21   format(2x,i2,2x,2(i2,2x,f9.6,2x),5x,'sr=',e14.6)
      return
      end


           function vn(n1,l1,z1,n2,l2,z2)
c-------------- < f(n1,l1,m1,z1,r) / f(n2,l2,m2,z2,r) > ----------
c   f(n,l,m,z,r) - hudrogen - like  function  in  charge  z
c-------------------------------------------------------------------
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      vn=0.0
      if(l1.ne.l2) return
      if(dabs(z1-z2).lt.0.00001) then
         if(n1.eq.n2)  vn=1.0
      return
      endif
c
      im1=n1-l1
      im2=n2-l2
      if(im1.le.0.or.im2.le.0) write(*,21) n1,l1,im1,n2,l2,im2
 21   format(5x,'error in vn: ', i3,i3,i3,5x,i3,i3,i3)
      y1=((2.0*z1/n1)**(l1+1))*dsqrt(z1*fn(n1-l1-1)/(fn(n1+l1)**3))/n1
      y2=((2.0*z2/n2)**(l2+1))*dsqrt(z2*fn(n2-l2-1)/(fn(n2+l2)**3))/n2
                 p1=0.0
             do 2 i1=1,im1
      c1=(1.-2.*dfloat(i1-1-((i1-1)/2)*2))*((2.*z1/n1)**(i1-1))*
     &     (fn(n1+l1)**2)/(fn(n1-l1-i1)*fn(2*l1+i1)*fn(i1-1))
             p2=0.0
      do 3 i2=1,im2
      c2=(1.-2.*dfloat(i2-1-((i2-1)/2)*2))*((2.*z2/n2)**(i2-1))*
     &     (fn(n2+l2)**2)/(fn(n2-l2-i2)*fn(2*l2+i2)*fn(i2-1))
      p2=p2+c2*fn(l1+l2+i1+i2)/(z1/n1+z2/n2)**(l1+l2+i1+i2+1)
  3   continue
         p1=p1+p2*c1
  2      continue
      vn=y1*y2*p1
      return
      end
                        function fn(k)
c-----------------------   fn(k)=k! --------------------
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      if(k.lt.0) write(*,21) k
  21  format(5x,'error in fn(k) function  k=',i5)
      fn=1.0
      if(k.le.1) return
      do 1 i=1,k
   1  fn=fn*dfloat(i)
      return
      end

                function  clb(l1,m1,l2,m2,l3,m3)
c--- < l1,m1,l2,m2 / l3, m3 >  (cm.  L.F. p.510 or Messia-2 p.527)   --------
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      clb=0.0
      if(m1+m2.ne.m3 )  return
      if(iabs(m1).gt.l1.or.iabs(m2).gt.l2.or.iabs(m3).gt.l3) return
      if(l1.gt.l2+l3.or. l2.gt.l1+l3 .or. l3.gt.l1+l2)  return
c--
      y=dsqrt(dfloat(2*l3+1)*fn(l1+l2-l3)*fn(l2+l3-l1)*fn(l3+l1-l2)
     &  *fn(l1+m1)*fn(l1-m1)*fn(l2+m2)*fn(l2-m2)*fn(l3+m3)*fn(l3-m3)
     &  /fn(l1+l2+l3+1))
      sum=0.0
      i=-1
  1   i=i+1
      if(l3-l2+m1+i.lt.0.or.l3-l1-m2+i.lt.0) goto 1
c------------ (-1)**i=((i+1)-2*((i+1)/2))-(i-2*(i/2))  --------------
      sum=sum+
     &  dfloat(((i+1)-2*((i+1)/2))-(i-2*(i/2)))/(fn(i)*fn(l3-l2+m1+i)
     &    *fn(l3-l1-m2+i)*fn(l1+l2-l3-i)*fn(l1-m1-i)*fn(l2+m2-i))
      if(l1-m1-i.ge.1.and.l2+m2-i.ge.1.and.l1+l2-l3-i.ge.1) goto 1
      clb=y*sum
      return
      end

         subroutine vnort(vif)
c----------------------------------------------------------------
c               vif = < nlf function / nli function >
c----------------------------------------------------------------
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      parameter   ( jxne=3, jcmain=50 )
      common/fmain/ zt,gx(jcmain),ax(jxne,jcmain),
     &  ns(jxne,jcmain),ls(jxne,jcmain),kmain,ne,nspin,lres,iprint
      common/wmain/ zw,gw(jcmain),aw(jxne,jcmain),
     &  nw(jxne,jcmain),lw(jxne,jcmain),kwmax,new,nspinw,lresw
      data  eps/1.e-5/

      vif=0.0
      sign=1.0
      if(nspin.eq.3) sign=-1.0

      do 2 i=1,kwmax
      ni1=nw(1,i)
      li1=lw(1,i)
      ni2=nw(2,i)
      li2=lw(2,i)
      zi1=aw(1,i)
      zi2=aw(2,i)
c     wi=gw(i)

      yi=1.0/dsqrt(2.d0)
      if(ni1.eq.ni2.and.li1.eq.li2.and.dabs(zi1-zi2).lt.eps) yi=0.5
      do 1 j=1,kmain
      nf1=ns(1,j)
      lf1=ls(1,j)
      nf2=ns(2,j)
      lf2=ls(2,j)
      zf1=ax(1,j)
      zf2=ax(2,j)
c      wf=gx(j)
      yf=1.0/dsqrt(2.d0)
      if(nf1.eq.nf2.and.lf1.eq.lf2
     &   .and. dabs(zf1- zf2).lt.eps  )   yf=0.5
c
      vif=vif+2.*yf*yi*gw(i)*gx(j)*
     & (  vn(nf1,lf1,zf1,ni1,li1,zi1)
     &  * vn(nf2,lf2,zf2,ni2,li2,zi2)
     &  + vn(nf1,lf1,zf1,ni2,li2,zi2)
     &  * vn(nf2,lf2,zf2,ni1,li1,zi1) * sign )
  1   continue
  2   continue
      return
      end

      subroutine energ(enr,rnm1)
      implicit real*8 (a-h,o-z), integer*4 (i-n)
      parameter   ( jxne=3, jcmain=50 )
      common/cnsrak/  c9j(3,5,5,9,5,5)
      common/fmain/ zt,gx(jcmain),ax(jxne,jcmain),
     &  ns(jxne,jcmain),ls(jxne,jcmain),kmain,ne,nspin,lres,iprint
       dimension     gi(jcmain),axi(jxne,jcmain),
     &         nsi(jxne,jcmain),lsi(jxne,jcmain),
     &               gf(jcmain),axf(jxne,jcmain),
     &         nsf(jxne,jcmain),lsf(jxne,jcmain)

      data delta/1.e-6/, eps/1.e-12/, mm1/-1/, mm2/-2/
      sign=1.0
      if(nspin.eq.3) sign=-1.0
c------------------ equivalence initial and final state -------------
      nimax=kmain
      nfmax=kmain
      do 101  n=1,ne
      do 102  k=1,kmain
      nsi(n,k)=ns(n,k)
      lsi(n,k)=ls(n,k)
      axi(n,k)=ax(n,k)
      gi(k)=gx(k)
      nsf(n,k)=ns(n,k)
      lsf(n,k)=ls(n,k)
      axf(n,k)=ax(n,k)
      gf(k)=gx(k)
 102  continue
 101  continue


      smg1=0.0
      rnm1=0.0
      do  8  ngi=1,nimax
      if( dabs(gi(ngi)).lt.eps ) goto 8
      zi1=axi(1,ngi)
      zi2=axi(2,ngi)
      ni1=nsi(1,ngi)
      li1=lsi(1,ngi)
      ni2=nsi(2,ngi)
      li2=lsi(2,ngi)
      imax1=ni1-li1
      imax2=ni2-li2
c
      yi1=((2.0*zi1/ni1)**(li1+1))
     &      * dsqrt(zi1*fn(ni1-li1-1)/(fn(ni1+li1)**3))/ni1
      yi2=((2.0*zi2/ni2)**(li2+1))
     &      * dsqrt(zi2*fn(ni2-li2-1)/(fn(ni2+li2)**3))/ni2
      yyi=1./dsqrt(2.d0)
      if(dabs(zi1-zi2).le.delta.and.ni1.eq.ni2
     &                         .and.li1.eq.li2)  yyi=0.5
           smg2=0.0
           rnm2=0.0
           do  9  ngf=1,nfmax
           if( dabs(gf(ngf)).lt.eps ) goto 9
           zf1=axf(1,ngf)
           zf2=axf(2,ngf)
           nf1=nsf(1,ngf)
           lf1=lsf(1,ngf)
           nf2=nsf(2,ngf)
           lf2=lsf(2,ngf)
           jmax1=nf1-lf1
           jmax2=nf2-lf2
c
      yf1=((2.0*zf1/nf1)**(lf1+1))
     &      * dsqrt(zf1*fn(nf1-lf1-1)/(fn(nf1+lf1)**3))/nf1
      yf2=((2.0*zf2/nf2)**(lf2+1))
     &      * dsqrt(zf2*fn(nf2-lf2-1)/(fn(nf2+lf2)**3))/nf2
      yyf=1./dsqrt(2.d0)
      if(dabs(zf1-zf2).le.delta.and.nf1.eq.nf2
     &                           .and.lf1.eq.lf2)  yyf=0.5

      sumi1=0.0
      rumi1=0.0
      do 1 i1=1,imax1
      ci1=(1.-2.*dfloat(i1-1-((i1-1)/2)*2))
     &    *((2.*zi1/ni1)**(i1-1))
     &    *(fn(ni1+li1)**2)/(fn(ni1-li1-i1)*fn(2*li1+i1)*fn(i1-1))
c
      if(iprint.ge.4) then
      write(15,31) i1,ni1,li1,zi1,yi1,ci1
  31  format(' i1=',i2,2x,2i1,2x,f9.6,3x,f7.3,3x,e12.4)
      endif
c
      sumi2=0.0
      rumi2=0.0
      do 2 i2=1,imax2
      ci2=(1.-2.*dfloat(i2-1-((i2-1)/2)*2))
     &    *((2.*zi2/ni2)**(i2-1))
     &    *(fn(ni2+li2)**2)/(fn(ni2-li2-i2)*fn(2*li2+i2)*fn(i2-1))
c
      if(iprint.ge.4) then
      write(15,32) i2,ni2,li2,zi2,yi2,ci2
  32  format(5x,'i2=',i2,2x,2i1,2x,f9.6,3x,f7.3,3x,e12.4)
      endif

      sumf1=0.0
      rumf1=0.0
      do 11 j1=1,jmax1
      cf1=(1.-2.*dfloat(j1-1-((j1-1)/2)*2))
     &    *((2.*zf1/nf1)**(j1-1))
     &    *(fn(nf1+lf1)**2)/(fn(nf1-lf1-j1)*fn(2*lf1+j1)*fn(j1-1))
c
      if(iprint.ge.4) then
      write(15,41) j1,nf1,lf1,zf1,yf1,cf1
  41  format(8x,'j1=',i2,2x,2i1,2x,f9.6,3x,f7.3,3x,e12.4)
      endif
c
           z11=zi1/dfloat(ni1) + zf1/dfloat(nf1)
           z21=zi2/dfloat(ni2) + zf1/dfloat(nf1)
           j11=li1+i1+lf1+j1
           j21=li2+i2+lf1+j1

      sumf2=0.0
      rumf2=0.0
      do 12 j2=1,jmax2
      cf2=(1.-2.*dfloat(j2-1-((j2-1)/2)*2))
     &    *((2.*zf2/nf2)**(j2-1))
     &    *(fn(nf2+lf2)**2)/(fn(nf2-lf2-j2)*fn(2*lf2+j2)*fn(j2-1))
c
      if(iprint.ge.4) then
      write(15,42) j2,nf2,lf2,zf2,yf2,cf2
  42  format(12x,'j2=',i2,2x,2i1,2x,f9.6,3x,f7.3,3x,e12.4)
      endif
c
           z12=zi1/dfloat(ni1) + zf2/dfloat(nf2)
           z22=zi2/dfloat(ni2) + zf2/dfloat(nf2)
           j12=li1+i1+lf2+j2
           j22=li2+i2+lf2+j2
c
      v11=0.0
      v12=0.0
      v21=0.0
      v22=0.0
      if(li1.eq.lf1) v11=fn(j11)/(z11**(j11+1))
      if(li2.eq.lf1) v21=fn(j21)/(z21**(j21+1))
      if(li1.eq.lf2) v12=fn(j12)/(z12**(j12+1))
      if(li2.eq.lf2) v22=fn(j22)/(z22**(j22+1))

        lmin=iabs(li1-lf1) + 1
        if(iabs(li1-lf2)+1.lt.lmin) lmin=iabs(li1-lf2) + 1
        if(iabs(li2-lf1)+1.lt.lmin) lmin=iabs(li2-lf1) + 1
        if(iabs(li2-lf2)+1.lt.lmin) lmin=iabs(li2-lf2) + 1
c
        lmax=li1+lf1 + 1
        if(li1+lf2+1.gt.lmax) lmax=li1+lf2 + 1
        if(li2+lf1+1.gt.lmax) lmax=li2+lf1 + 1
        if(li2+lf2+1.gt.lmax) lmax=li2+lf2 + 1

c         ----------------------------------
           g1212=0.0
           g1221=0.0
      do 51 ll=lmin,lmax
      l=ll-1
      c1212=c9j(lres+1,li1+1,li2+1,l+1,lf1+1,lf2+1)
      c1221=c9j(lres+1,li1+1,li2+1,l+1,lf2+1,lf1+1)
      if( dabs(c1212).gt.eps)
     &  g1212=g1212 + 2.0 * c1212 * sr(l,j11-2,z11,j22-2,z22)
      if( dabs(c1221).gt.eps)
     &  g1221=g1221 + sign * 2.0* c1221 * sr(l,j12-2,z12,j21-2,z21)
 51   continue

c       *************************************
c                   NE=2
c       *************************************
c          ------------------  -zt/r1 term ---------------
      q11= fn(j11+mm1)/(z11**(j11+1+mm1)) * v22
      q12= fn(j12+mm1)/(z12**(j12+1+mm1)) * v21
      q21= fn(j21+mm1)/(z21**(j21+1+mm1)) * v12
      q22= fn(j22+mm1)/(z22**(j22+1+mm1)) * v11
c      ------------  l*(l+1)/(2.*r1*r1)  term ---------------
      w11=  fn(j11+mm2)/(z11**(j11+1+mm2)) * v22
      w12=  fn(j12+mm2)/(z12**(j12+1+mm2)) * v21
      w21=  fn(j21+mm2)/(z21**(j21+1+mm2)) * v12
      w22=  fn(j22+mm2)/(z22**(j22+1+mm2)) * v11
c      ------------- normirovka ---------------
      r11= v11*v22
      r12= v12*v21
      r22= v11*v22
      r21= v12*v21
             rr =0.0
             tw11=0.0
             tq11=0.0
             tx11=0.0
             tw22=0.0
             tq22=0.0
             tx22=0.0
      if(li1.eq.lf1.and.li2.eq.lf2) then
      tw11=tw11+ dfloat( li1*(li1+1) - (li1+i1-1)*(li1+i1) )/2. * w11
      tq11=tq11+ ( zi1*dfloat(li1+i1)/dfloat(ni1)  - zt)        * q11
      tx11=tx11-    zi1*zi1/(2.*dfloat(ni1*ni1))                * r11
      tw22=tw22+ dfloat( li2*(li2+1) - (li2+i2-1)*(li2+i2) )/2. * w22
      tq22=tq22+ ( zi2*dfloat(li2+i2)/dfloat(ni2)  - zt)        * q22
      tx22=tx22-    zi2*zi2/(2.*dfloat(ni2*ni2))                * r22
      rr= rr +  r11 + r22
      endif
c
             tw12=0.0
             tq12=0.0
             tx12=0.0
             tw21=0.0
             tq21=0.0
             tx21=0.0
      if(li1.eq.lf2.and.li2.eq.lf1) then
      tw12=tw12+ dfloat( li1*(li1+1) - (li1+i1-1)*(li1+i1) )/2. * w12
      tq12=tq12+ ( zi1*dfloat(li1+i1)/dfloat(ni1)  - zt)        * q12
      tx12=tx12-    zi1*zi1/(2.*dfloat(ni1*ni1))                * r12
      tw21=tw21+ dfloat( li2*(li2+1) - (li2+i2-1)*(li2+i2) )/2. * w21
      tq21=tq21+ ( zi2*dfloat(li2+i2)/dfloat(ni2)  - zt)        * q21
      tx21=tx21-    zi2*zi2/(2.*dfloat(ni2*ni2))                * r21
      rr= rr + (r12+r21) *sign
      endif

      hh =tw11 + tq11 + tx11 + tw22 + tq22 + tx22
     &  +(tw12 + tq12 + tx12 + tw21 + tq21 + tx21)*sign
      h=2.0*hh + (g1212 + g1221)
      sumf2=sumf2 + h   *cf2*yf2
      rumf2=rumf2 + rr  *cf2*yf2


 12   continue
      sumf1=sumf1+sumf2*cf1*yf1*yyf
      rumf1=rumf1+rumf2*cf1*yf1*yyf
 11   continue
      sumi2=sumi2+sumf1*ci2*yi2
      rumi2=rumi2+rumf1*ci2*yi2
  2   continue
      sumi1=sumi1+sumi2*ci1*yi1*yyi
      rumi1=rumi1+rumi2*ci1*yi1*yyi
  1   continue

           smg2=smg2+sumi1*gf(ngf)
           rnm2=rnm2+rumi1*gf(ngf)
      if(iprint.ge.3)  then
      write(15,38) ngi,ni1,li1,ni2,li2,gi(ngi),
     &             ngf,nf1,lf1,nf2,lf2,gf(ngf),sumi1,rumi1
 38   format(1x,i2,1x,4i1,1x,f9.6,5x,i2,1x,4i1,
     &                    1x,f9.6,5x,e12.4,2x,e12.4,2x,e12.4,/ )
      endif
  9        continue
      smg1=smg1+smg2*gi(ngi)
      rnm1=rnm1+rnm2*gi(ngi)
  8   continue
      enr=0.0
      if(dabs(rnm1).gt.eps)  enr=smg1/rnm1
      if(iprint.ge.3) write(15,39) enr,rnm1
 39   format(25x,'enr=',f16.10,5x,'rnm1=',f16.10)
      return
      end
